"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printHtmlNode = exports.printHtmlChildren = exports.pruneHtmlNode = exports.extractPseudoPlainPart = exports.normalizeVendorHtml = void 0;
const constants_1 = require("../constants");
const outlook_1 = require("./strategies/outlook");
const constants_2 = require("../constants");
const utils_1 = require("../utils");
const gmail_1 = require("./strategies/gmail");
const index_1 = require("../index");
const common_1 = require("./strategies/common");
const documentNormalizationFunctions = {
    [constants_2.EMAIL_VENDORS.GMAIL]: gmail_1.normalizeGmailDocument,
    [constants_2.EMAIL_VENDORS.OUTLOOK]: outlook_1.normalizeOutlookDocument,
};
const normalizeVendorHtml = (document, vendor) => {
    /**
     * Remove unnecessary nodes
     */
    exports.pruneHtmlNode(document.body);
    /**
     * Apply document normalisations
     */
    common_1.normalizeDocumentCommon(document.body);
    const normalizeDocument = documentNormalizationFunctions[vendor];
    if (normalizeDocument) {
        normalizeDocument(document);
    }
    /**
     * Final printout
     */
    return exports.printHtmlChildren(document.body, 0);
};
exports.normalizeVendorHtml = normalizeVendorHtml;
const extractPseudoPlainPart = (document) => {
    let normalizedTextContent = index_1.PlainNormalizer.normalizePlain(document.body.textContent);
    const anchors = document.getElementsByTagName("a");
    const images = document.getElementsByTagName("img");
    let urls = [];
    Array.from(anchors).forEach((a) => {
        urls.push(a.getAttribute("href"));
    });
    Array.from(images).forEach((img) => {
        urls.push(img.getAttribute("src"));
    });
    urls = urls.filter((attr) => !!attr).sort();
    normalizedTextContent += urls.join(",");
    return normalizedTextContent;
};
exports.extractPseudoPlainPart = extractPseudoPlainPart;
const pruneHtmlNode = (node) => {
    let toBeRemoved = false;
    switch (node.nodeType) {
        case constants_1.COMMENT_NODE:
        case constants_1.DOCUMENT_TYPE_NODE:
            toBeRemoved = true;
            break;
        case constants_1.ELEMENT_NODE:
            toBeRemoved = common_1.pruneElement(node);
            break;
    }
    if (toBeRemoved) {
        return true;
    }
    const childrenToRemove = [];
    let child = node.firstChild;
    while (child) {
        exports.pruneHtmlNode(child) && childrenToRemove.push(child);
        child = child.nextSibling;
    }
    childrenToRemove.forEach((child) => node.removeChild(child));
    return false;
};
exports.pruneHtmlNode = pruneHtmlNode;
const printHtmlChildren = (node, depth) => {
    let child = node.firstChild;
    if (!child) {
        return "";
    }
    if (child == node.lastChild && child.nodeType == constants_1.TEXT_NODE) {
        return exports.printHtmlNode(child, depth);
    }
    else {
        let result = "";
        while (child) {
            result = result.concat(exports.printHtmlNode(child, depth));
            child = child.nextSibling;
        }
        return result;
    }
};
exports.printHtmlChildren = printHtmlChildren;
const printHtmlNode = (node, depth) => {
    let result = "";
    switch (node.nodeType) {
        case constants_1.TEXT_NODE: {
            const text = utils_1.normalizeTextSpacings(node.textContent).trim();
            if (text.length) {
                result += "<TEXT>";
                result += text;
                result += "</TEXT>";
                result += "\n";
            }
            break;
        }
        case constants_1.DOCUMENT_NODE:
            result += exports.printHtmlChildren(node, depth);
            break;
        case constants_1.ELEMENT_NODE:
            result += "<" + node.nodeName;
            Array.from(node.attributes)
                .filter((a) => common_1.ATTRIBUTES_TO_KEEP[a.name])
                .sort((a, b) => a.name.localeCompare(b.name))
                .forEach((attribute) => {
                result += ` ${attribute.name}`;
                if (attribute.value) {
                    result += `="${attribute.value}"`;
                }
            });
            if (node.firstChild) {
                const printout = exports.printHtmlChildren(node, depth + 1);
                if (printout.trim().length === 0) {
                    result += "/>";
                }
                else {
                    result += ">";
                    result += "\n";
                    result += printout;
                    result += "</" + node.nodeName + ">";
                }
            }
            else {
                result += "/>";
            }
            result += "\n";
            break;
    }
    return result;
};
exports.printHtmlNode = printHtmlNode;
