// this is a Node module. require is a must to work across different envs
const URL = require("url-parse");

export const ELEMENT_TYPES_TO_REMOVE = {
  BR: true,
  HR: true,
  USE: true,
  SVG: true,
};

export const ATTRIBUTES_TO_KEEP = {
  alt: true,
  src: true,
  cite: true,
  data: true,
  datetime: true,
  href: true,
  value: true,
};

export const normalizeDocumentCommon = (body: HTMLElement): void => {
  /**
   * Unwind Outlook safelink wrappers
   */
  const anchors = body.getElementsByTagName("a");
  for (const anchor of anchors) {
    const url = new URL(anchor.getAttribute("href"), true);

    if (url.host.includes("safelinks.protection.outlook.com")) {
      anchor.setAttribute("href", url.query["url"]);
    }
  }

  /**
   * Unwind Gmail "googleusercontent" wrappers
   */
  const images = body.getElementsByTagName("img");
  for (const image of images) {
    let url;
    try {
      url = new URL(image.getAttribute("src"));
    } catch (e) {
      url = null;
    }

    if (url && url.host.includes("googleusercontent.com")) {
      const originalUrl = url.hash.slice(1, url.hash.length);
      if (originalUrl) {
        image.setAttribute("src", originalUrl);
      }
    }
  }
};

/**
 * Decides whether node should be removed
 * @param element
 */
export const pruneElement = (element: HTMLElement): boolean => {
  return !!ELEMENT_TYPES_TO_REMOVE[element.nodeName];
};

export const cloneAnchorFromPane = (
  a: HTMLAnchorElement,
  pane: HTMLElement
): void => {
  try {
    const url = new URL(a.getAttribute("href"));
    // If this is external url
    if (url.host && url.protocol) {
      pane.parentNode.insertBefore(a.cloneNode(false), pane);
    }
  } catch (e) {
    return e;
  }
};
