import { cloneAnchorFromPane } from "./common";
import { ELEMENT_NODE } from "../../constants";

const qrCodeContainerIds = { vereignWrapperLink: 1 };
const removeQrCodeNodes = (document: HTMLDocument) => {
  const remove = (node: Element) => {
    let toRemove = [];

    let child = node.firstChild;

    while (child) {
      if (child.nodeType == ELEMENT_NODE) {
        toRemove = [...toRemove, ...remove(child as Element)];

        const childElement = child as Element;
        const id = childElement.getAttribute("id");
        if (
          id &&
          Object.keys(qrCodeContainerIds).find((possibleId) =>
            id.includes(possibleId)
          )
        ) {
          toRemove.push(childElement);
        }
      }

      child = child.nextSibling;
    }

    return toRemove;
  };

  const elementsToRemove = remove(document.body);
  elementsToRemove.forEach((element) =>
    element.parentNode.removeChild(element)
  );
};

export const normalizeGmailDocument = (document: HTMLDocument): void => {
  removeQrCodeNodes(document);

  /**
   * Look for attachments panes and extract <a> tags from them
   */
  const attachmentsPanes = Array.from(
    document.getElementsByClassName("gmail_chip")
  );

  attachmentsPanes.forEach((pane) => {
    const as = pane.getElementsByTagName("a");
    Array.from(as).forEach((a) => {
      cloneAnchorFromPane(a, pane as HTMLElement);
    });
  });

  attachmentsPanes.forEach((pane) => {
    pane.parentNode.removeChild(pane);
  });
};
