export const unwindTags = (nodes: Array<Node>): void => {
  //Sort nodes by depth to start unwinding the deepest ones
  const tagsDepths: { depth?: Array<Node> } = {};
  Array.from(nodes).forEach((span: Node) => {
    let descendant = span;
    let parent = descendant.parentNode;

    let depth = 0;
    while (parent && descendant !== parent) {
      descendant = parent;
      parent = descendant.parentNode;
      depth++;
    }

    if (!tagsDepths[depth]) {
      tagsDepths[depth] = [];
    }

    tagsDepths[depth].push(span);
  });

  Object.keys(tagsDepths)
    .sort((a, b) => parseInt(b) - parseInt(a))
    .forEach((depth) => {
      tagsDepths[depth].forEach((span) => {
        let child = span.firstChild;
        const parent = span.parentNode;
        while (child) {
          parent.insertBefore(child.cloneNode(true), span);

          child = child.nextSibling;
        }

        span.parentNode.removeChild(span);
      });
    });
};
