import { ELEMENT_NODE, TEXT_NODE } from "../../constants";
import { cloneAnchorFromPane } from "./common";
import { unwindTags } from "./nodesAmendingFunctions";

const qrCodeContainerIds = {
  "test-for-us": 1,
};
const removeQrCodeNodes = (document: HTMLDocument) => {
  const remove = (node: Element) => {
    let toRemove = [];

    let child = node.firstChild;

    while (child) {
      if (child.nodeType == ELEMENT_NODE) {
        toRemove = [...toRemove, ...remove(child as Element)];

        const childElement = child as Element;
        const id = childElement.getAttribute("id");
        if (
          id &&
          Object.keys(qrCodeContainerIds).find((possibleId) =>
            id.includes(possibleId)
          )
        ) {
          toRemove.push(childElement.parentNode);
        }
      }

      child = child.nextSibling;
    }

    return toRemove;
  };

  const elementsToRemove = remove(document.body);
  elementsToRemove.forEach((element) =>
    element.parentNode.removeChild(element)
  );
};

export const normalizeOutlookDocument = (document: HTMLDocument): void => {
  /**
   * Remove QR code entries
   */
  removeQrCodeNodes(document);

  /**
   * Unwind Word o:p paragraphs
   */
  const ops = document.getElementsByTagName("o:p");
  unwindTags(Array.from(ops));

  /**
   * Remove empty paragraphs
   */
  const ps = document.getElementsByTagName("p");
  Array.from(ps).forEach((p) => {
    if (p.childNodes.length === 0) {
      p.parentNode.removeChild(p);
    }

    if (p.childNodes.length === 1 && p.childNodes[0].nodeType === TEXT_NODE) {
      const text = p.childNodes[0].textContent;
      if (!text.replace(/\u00A0/g, "").trim()) {
        p.parentNode.removeChild(p);
      }
    }
  });

  /**
   * Unwind all MSONormal, because outlook might wrap them into <div></div>
   */
  const msoNormalWrappers = document.getElementsByClassName("MsoNormal");
  const msoNormalParents = Array.from(msoNormalWrappers)
    .map((node) => node.parentNode)
    .filter((node, index, self) => self.indexOf(node) === index);
  unwindTags(msoNormalParents);

  /**
   * Unwind WordSection1 tags
   */
  const wordSectionWrappers = document.getElementsByClassName("WordSection1");
  unwindTags(Array.from(wordSectionWrappers));

  /**
   * Get rid of attachments panes
   */
  const attachmentsPanesContainer = document.getElementById(
    "OwaReferenceAttachments"
  );
  const attachmentsPanesContainerEnd = document.getElementById(
    "OwaReferenceAttachmentsEnd"
  );

  if (attachmentsPanesContainer) {
    const as = attachmentsPanesContainer.getElementsByTagName("a");
    Array.from(as)
      .sort((a, b) =>
        a.getAttribute("href").localeCompare(b.getAttribute("href"))
      )
      .forEach((a) => {
        cloneAnchorFromPane(a, attachmentsPanesContainer as HTMLElement);
      });

    attachmentsPanesContainer.parentNode.removeChild(attachmentsPanesContainer);
  }

  attachmentsPanesContainerEnd &&
    attachmentsPanesContainerEnd.parentNode.removeChild(
      attachmentsPanesContainerEnd
    );

  /**
   * Unwind spans, because sometimes Outlook wraps everything into span after sending
   */
  const spans = document.getElementsByTagName("span");
  unwindTags(Array.from(spans));
};
