// this is a Node module. require is a must to work across different envs
const URL = require("url-parse");

import { normalizeTextSpacings } from "../utils";

export const normalizePlainPart = (text: string): string => {
  text = cleanupHiddenCharacters(text);
  text = removeQRCodes(text);
  text = normalizeTextSpacings(text);
  text = patchOutlookSafelinksWrappers(text);

  return text.trim();
};

const patchOutlookSafelinksWrappers = (text: string) => {
  const links = text.match(
    /<https:.+?(safelinks\.protection\.outlook\.com).+?>/g
  );

  if (links) {
    links.forEach((link) => {
      const url = new URL(link.slice(1, link.length - 1), true);
      const originalUrl = url.query["url"];
      text = text.replace(link, `<${originalUrl}>`);
    });
  }

  return text;
};

const removeQRCodes = (s: string): string => {
  return s
    .replace(
      /\[(image:\s)*qrcode.png]\s*<https:\/\/.+(vereign\.com|vrgnservices\.com).*?>/g,
      ""
    )
    .replace(
      /<https:\/\/.+(vereign\.com|vrgnservices\.com).*?>\s*\[(image: )*qrcode.png]/g,
      ""
    );
};

export const cleanupHiddenCharacters = (s: string): string => {
  const removeSymbols = new RegExp(/[\u200B]+/g);
  return s.replace(removeSymbols, "");
};
